﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;
using BeyondBasicUnitTesting.Business;

namespace BeyondBasicUnitTesting.UnitTests
{
    [TestClass]
    public class PersonPresenterFixture : UnitTestBase
    {
        private IRepository<IPerson> m_personRepositoryInstance;
        protected IRepository<IPerson> PersonRepositoryInstance
        {
            get
            {
                if (m_personRepositoryInstance == null)
                {
                    m_personRepositoryInstance = new RepositoryMock<IPerson>();
                }

                return m_personRepositoryInstance;
            }
        }

        private ILogger m_loggerInstance;
        protected ILogger LoggerInstance
        {
            get
            {
                if (m_loggerInstance == null)
                {
                    m_loggerInstance = new Logger();
                }

                return m_loggerInstance;
            }
        }

        private IPersonPresenter m_presenter;
        protected IPersonPresenter Presenter
        {
            get
            {
                if (m_presenter == null)
                {
                    m_presenter = new PersonPresenter(LoggerInstance, PersonRepositoryInstance);
                }

                return m_presenter;
            }
        }

        private IList<IPerson> m_testPersons;
        public IList<IPerson> TestPersons
        {
            get
            {
                if (m_testPersons == null)
                {
                    FacadeInstance.SavePerson(FacadeInstance.Create("Ben", "Day", "benday@benday.com", "ACTIVE")); 
                    FacadeInstance.SavePerson(FacadeInstance.Create("Grant", "Achatz", "grant@alinea.com", "ACTIVE"));
                    FacadeInstance.SavePerson(FacadeInstance.Create("Thomas", "Keller", "t.keller@frenchlaundry.com", "ACTIVE"));
                    FacadeInstance.SavePerson(FacadeInstance.Create("Tony", "Maws", "tony@craigieonmain.com", "ACTIVE"));
                    FacadeInstance.SavePerson(FacadeInstance.Create("Michael", "Schlow", "shlow@viamattarestaurant.com", "ACTIVE"));

                    m_testPersons = new List<IPerson>(); 
                        
                    var personsFromRepository = PersonRepositoryInstance.GetAll();

                    foreach (var item in personsFromRepository)
                    {
                        m_testPersons.Add(item);
                    }
                }

                return m_testPersons;
            }
        }

        private MockRepository m_mockRepositoryInstance;
        public MockRepository MockRepositoryInstance
        {
            get
            {
                if (m_mockRepositoryInstance == null)
                {
                    m_mockRepositoryInstance = new MockRepository();
                }

                return m_mockRepositoryInstance;
            }
        }

        private IPersonFacade m_facadeInstance;
        public IPersonFacade FacadeInstance
        {
            get
            {
                if (m_facadeInstance == null)
                {
                    m_facadeInstance = new PersonFacade(LoggerInstance, PersonRepositoryInstance);
                }

                return m_facadeInstance;
            }
        }

        [TestMethod]
        public void PersonPresenter_SearchInitializeBlank()
        {
            IPersonSearchView view = MockRepositoryInstance.Stub<IPersonSearchView>();

            Presenter.InitializeBlank(view);

            Assert.IsNull(view.Results, "Results collection should be null.");

            IList<IPerson> list = new List<IPerson>();


            view.Results = list;

            Presenter.InitializeBlank(view);

            Assert.IsNull(view.Results, "Results collection should be null.");
        }

        [TestMethod]
        public void PersonPresenter_SearchViewAllNoResults()
        {
            IPersonSearchView view = MockRepositoryInstance.Stub<IPersonSearchView>();

            Presenter.InitializeBlank(view);

            Presenter.ViewAll(view);

            Assert.IsNotNull(view.Results, "Results was null.");
            Assert.AreEqual<int>(0, view.Results.Count, "Result count should be 0.");
        }

        private void AssertAreEqual(IList<IPerson> expected, IList<IPerson> actual)
        {
            Assert.IsNotNull(expected, "Expected was null.");
            Assert.IsNotNull(actual, "Actual was null.");

            Assert.AreEqual<int>(expected.Count, actual.Count, "Count was wrong.");

            for (int i = 0; i < expected.Count; i++)
            {
                AssertAreEqual(expected[i], actual[i]);
            }
        }

        private void AssertAreEqual(IPerson expected, IPerson actual)
        {
            Assert.IsNotNull(expected, "Expected was null.");
            Assert.IsNotNull(actual, "Actual was null.");

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.FirstName, actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName, actual.LastName, "LastName");
            Assert.AreEqual<string>(expected.EmailAddress, actual.EmailAddress, "EmailAddress");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");
        }

        [TestMethod]
        public void PersonPresenter_SearchViewAllWithResults()
        {
            int expectedPersonCount = TestPersons.Count;

            Assert.AreNotEqual<int>(0, expectedPersonCount, "No person test data records.");

            IPersonSearchView view = MockRepositoryInstance.Stub<IPersonSearchView>();

            Presenter.InitializeBlank(view);

            Presenter.ViewAll(view);

            Assert.IsNotNull(view.Results, "Results was null.");
            Assert.AreEqual<int>(expectedPersonCount, view.Results.Count, "Result count should be 0.");

            AssertAreEqual(TestPersons, view.Results);
        }

        [TestMethod]
        public void PersonPresenter_DetailViewInitializeBlank()
        {
            IPersonDetailView view = MockRepositoryInstance.Stub<IPersonDetailView>();

            Presenter.InitializeBlank(view);

            Assert.AreEqual<int>(0, view.PersonId, "Id should be 0.");
            Assert.AreEqual<string>(String.Empty, view.FirstName, "FirstName");
            Assert.AreEqual<string>(String.Empty, view.LastName, "LastName");
            Assert.AreEqual<string>(String.Empty, view.EmailAddress, "EmailAddress");
            Assert.AreEqual<string>("ACTIVE", view.Status, "Status");
        }

        [TestMethod]
        public void PersonPresenter_GetPersonDetailById()
        {
            IPerson expectedPerson = TestPersons[0];

            IPersonDetailView view = MockRepositoryInstance.Stub<IPersonDetailView>();

            view.PersonId = expectedPerson.Id;

            Presenter.LoadById(view);

            AssertAreEqual(expectedPerson, view);
        }

        private void AssertAreEqual(IPerson expected, IPersonDetailView actual)
        {
            Assert.IsNotNull(expected, "Expected was null.");
            Assert.IsNotNull(actual, "Actual was null.");

            Assert.AreEqual<int>(expected.Id, actual.PersonId, "Id");
            Assert.AreEqual<string>(expected.FirstName, actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName, actual.LastName, "LastName");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");
            Assert.AreEqual<string>(expected.EmailAddress, actual.EmailAddress, "EmailAddress");
        }

        [TestMethod]
        public void PersonPresenter_CreateNewPersonViaUI()
        {
            IPersonDetailView view = MockRepositoryInstance.Stub<IPersonDetailView>();

            Presenter.InitializeBlank(view);

            view.FirstName = "Skip";
            view.LastName = "McJimbo";
            view.EmailAddress = "skip.mcjimbo@hotmail.com";

            Presenter.Save(view);

            Assert.AreNotEqual<int>(0, view.PersonId, "Id should not be 0 after save.");

            IPerson reloaded = PersonRepositoryInstance.GetById(view.PersonId);

            AssertAreEqual(reloaded, view);
        }

        [TestMethod]
        public void PersonPresenter_ModifyPersonViaUI()
        {
            IPerson original = TestPersons[0];

            IPersonDetailView view = MockRepositoryInstance.Stub<IPersonDetailView>();

            view.PersonId = original.Id;

            Presenter.LoadById(view);

            view.FirstName = "Skip";
            view.LastName = "McJimbo";
            view.EmailAddress = "skip.mcjimbo@hotmail.com";

            Presenter.Save(view);

            Assert.AreNotEqual<int>(0, view.PersonId, "Id should not be 0 after save.");

            IPerson reloaded = PersonRepositoryInstance.GetById(view.PersonId);

            AssertAreEqual(reloaded, view);
        }
    }
}
